compilestyle:

PRINT "Enter style name to generate: ";
INPUT "", inname$

inname$ = ls_strip_spaces$(inname$)

PRINT "Loading INI file to memory..."

IF inname$ = "" THEN
  IF _FILEEXISTS("style.ini") = false THEN
    PRINT "ERROR: "; inname$; ".ini not found!"
    END
  END IF
  ininum = lfc_load_ini("style.ini")
ELSE
  IF _FILEEXISTS(inname$ + ".ini") = false THEN
    PRINT "ERROR: "; inname$; ".ini not found!"
    END
  END IF
  ininum = lfc_load_ini(inname$ + ".ini")
  inname$ = inname$ + "_"
END IF

PRINT "  INI file loaded."

stidnum$ = lfc_get_ini_value$(ininum, "style_number")
stidnam$ = ""
IF stidnum$ = "" OR lfc_str$(VAL(stidnum$)) <> stidnum$ THEN
  if lfc_get_ini_value$(ininum, "style_name") = "" then
    stidnum$ = "X"
    PRINT "No style number defined."
    PRINT "Saving as groundXo.dat / vgagrX.dat."
  ELSE
    stidnam$ = lfc_get_ini_value$(ininum, "style_name")
	Print "Style name defined."
	Print "Saving as g_" + stidnam$ + ".dat / v_" + stidnam$ + ".dat."
  END IF
ELSE
  PRINT "Style number " + stidnum$ + "."
  PRINT "Saving as ground"; stidnum$; "o.dat / vgagr"; stidnum$; ".dat."
END IF

if stidnam$ = "" then
  grfilename$ = inname$ + "ground" + stidnum$ + "o.dat"
else
  grfilename$ = "g_" + stidnam$ + ".dat"
end if

grfile = lfc_new_file(grfilename$)
lfc_small_endian grfile

PRINT "Loading palette..."

palrgb = VAL(lfc_get_ini_value$(ininum, "rgb_images"))

SELECT CASE LCASE$(lfc_get_ini_value$(ininum, "palette_info"))
  CASE "bmp"
    PRINT "    Style uses BMP file to define palette."
		for ex = 0 to 2
		  fn$ = inname$ + "palette." + formatlist$(ex)
			if _fileexists(fn$) then exit for
		next
    pimgnum = lfc_load_image(fn$)
    palels = lfc_images(pimgnum, 0) - 1
    FOR x = 0 TO palels
      PRINT "    Loading color "; lfc_str$(x); "..."
      lfg_active_palette(x).vga_red = INT(lfc_get_pixel_red(pimgnum, x, 0) / 4)
      lfg_active_palette(x).vga_green = INT(lfc_get_pixel_green(pimgnum, x, 0) / 4)
      lfg_active_palette(x).vga_blue = INT(lfc_get_pixel_blue(pimgnum, x, 0) / 4)
      lfg_active_palette(x).ega_value = lfg_ega_color(lfc_get_pixel_red(pimgnum, x, 0), lfc_get_pixel_green(pimgnum, x, 0), lfc_get_pixel_blue(pimgnum, x, 0))
    NEXT
    lfc_close_image pimgnum
  CASE "24"
    PRINT "    Style uses INI-defined 24-bit colors."
    palels = VAL(lfc_get_ini_value$(ininum, "palette_colors")) - 1
    FOR x = 0 TO palels
      PRINT "    Loading color "; lfc_str$(x); "..."
      a$ = lfc_get_ini_value$(ininum, "palette_" + lfc_str$(0))
      lfg_active_palette(x).vga_red = INT(VAL(LEFT$(a$, 3)) / 4)
      lfg_active_palette(x).vga_green = INT(VAL(MID$(a$, 4, 3)) / 4)
      lfg_active_palette(x).vga_blue = INT(VAL(RIGHT$(a$, 3)) / 4)
      lfg_active_palette(x).ega_value = lfg_ega_color(lfg_active_palette(x).vga_red * 4, lfg_active_palette(x).vga_green * 4, lfg_active_palette(x).vga_blue * 4)
    NEXT
  CASE "18"
    PRINT "    Style uses INI-defined 18-bit colors."
    palels = VAL(lfc_get_ini_value$(ininum, "palette_colors")) - 1
    FOR x = 0 TO palels
      PRINT "    Loading color "; lfc_str$(x); "..."
      a$ = lfc_get_ini_value$(ininum, "palette_" + lfc_str$(0))
      lfg_active_palette(x).vga_red = INT(VAL(LEFT$(a$, 3)))
      lfg_active_palette(x).vga_green = INT(VAL(MID$(a$, 4, 3)))
      lfg_active_palette(x).vga_blue = INT(VAL(RIGHT$(a$, 3)))
      lfg_active_palette(x).ega_value = lfg_ega_color(lfg_active_palette(x).vga_red * 4, lfg_active_palette(x).vga_green * 4, lfg_active_palette(x).vga_blue * 4)
    NEXT
  CASE ELSE
    PRINT "ERROR: Palette type must be BMP, 18 or 24."
    END
END SELECT
lfr_save_palette grfile
PRINT "  Palette successfully loaded."

if palrgb = 1 then
  palbit = 19
else
  palbit = 4
  if palels = 31 then palbit = 5
end if

IF palels < 16 THEN
  FOR x = 16 TO 31
    lfg_active_palette(x) = lfg_active_palette(x - 16)
  NEXT
END IF

PRINT "Encoding terrain pieces..."
tgfile = lfc_new_file(inname$ + "vgagrX.dat.0")
tgposit = 0
aso = 0
DIM putterdat AS lfr_terrain_data
FOR dopiece = 0 TO 127
  for ex = 0 to 2
	  fn$ = inname$ + "t" + lfc_pad_str$(dopiece, 2) + "." + formatlist$(ex)
		if _fileexists(fn$) then exit for
	next
  IF ex = 3 THEN
    IF dopiece = 0 THEN
      PRINT "ERROR: No terrain pieces found!"
      END
    ELSE
      PRINT "  All terrain pieces encoded."
      EXIT FOR
    END IF
  END IF
  PRINT "  Encoding piece "; lfc_str$(dopiece); "..."
  tim = lfc_load_image(fn$)
  if dbmode = 1 then Print "  Image loaded"
  IF lfc_images(tim, 0) MOD 8 <> 0 THEN
    PRINT "ERROR: Terrain piece "; lfc_str$(dopiece); " has a width not divisible by 8!"
    END
  END IF
  putterdat.width = lfc_images(tim, 0)
  putterdat.height = lfc_images(tim, 1)
  putterdat.image_loc = tgposit mod 65536
	putterdat.unknown1 = int(tgposit / 65536) * 2
  tgposit = lfg_encode_image(tim, tgfile, tgposit, palbit, 0)
  putterdat.mask_loc = tgposit mod 65536
	putterdat.unknown1 = putterdat.unknown1 + (int(tgposit / 65536) * 256)
  IF VAL(lfc_get_ini_value$(ininum, "terrain_" + lfc_str$(dopiece) + "_steel")) <> 0 THEN
    putterdat.unknown1 = putterdat.unknown1 + 1
    aso = 1
  END IF
  if dbmode = 1 then Print "  Stats determined"
  tgposit = lfg_encode_image(tim, tgfile, tgposit, 1, 1)
  if dbmode = 1 then Print "  Image encoded"
  lfr_put_terrain grfile, dopiece, putterdat
  if dbmode = 1 then Print "  Applied to VGAGR section"
  lfc_close_image tim
  if dbmode = 1 then Print "  Image closed"
NEXT

PRINT "Encoding objects..."
ogfile = lfc_new_file(inname$ + "vgagrX.dat.1")
ogposit = 0
DIM putterdat2 AS lfr_object_data
FOR doobj = 0 TO 31
  lfr_set_defaults putterdat2
  putterdat2.end_animation_frame_index = VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_frames"))
  IF putterdat2.end_animation_frame_index <= 0 THEN
    IF doobj < 2 THEN
      PRINT "ERROR: Graphic sets require at least two objects!"
      END
    END IF
    PRINT "  All objects encoded."
    EXIT FOR
  END IF
  
  if VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_horizontal_frames")) = 1 then
    horzframe = 1
  else
    horzframe = 0
  end if
	
	for ex = 0 to 2
	  fn$ = inname$ + "o" + lfc_pad_str$(doobj, 2) + "." + formatlist$(ex)
		if _fileexists(fn$) then exit for
	next	
	
  IF _FILEEXISTS(fn$) = false THEN
    PRINT "ERROR: Image file not found for object "; lfc_str$(doobj); "!"
    END
  END IF
  putterdat2.start_animation_frame_index = VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_start_frame"))
  putterdat2.trigger_effect_id = VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_type"))
  IF putterdat2.trigger_effect_id <> 0 or doobj = 1 THEN
    putterdat2.trigger_left = INT((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_left")) + 3) / 4)
    putterdat2.trigger_top = INT((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_top")) + 3) / 4) + 1
    putterdat2.trigger_width = INT((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_width")) + 3) / 4)
    putterdat2.trigger_height = INT((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_height")) + 3) / 4)
    putterdat2.trap_sound_effect_id = VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_sound"))
  END IF

  txoff = (4 - ABS((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_left"))) MOD 4)) MOD 4
  tyoff = (4 - ABS((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_top"))) MOD 4)) MOD 4
  twoff = (4 - ABS((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_width"))) MOD 4)) MOD 4
  thoff = (4 - ABS((VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_height"))) MOD 4)) MOD 4

  putterdat2.unknown3 = thoff + (twoff * 4) + (tyoff * 16) + (txoff * 64)
  putterdat2.unknown3 = putterdat2.unknown3 + (VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_key_frame")) * 256)
	
	IF putterdat2.unknown3 <> 0 THEN adjusttriggers = 1
	
  putterdat2.trigger_px = VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_point_x"))
  putterdat2.trigger_py = VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_point_y"))

  putterdat2.animation_frames_base_loc = ogposit mod 65536
	putterdat2.unknown1 = int(ogposit / 65536)

  IF putterdat2.end_animation_frame_index > 1 THEN putterdat2.animation_flags = 2
  IF VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_anim")) <> 0 THEN putterdat2.animation_flags = 1

  oim = lfc_load_image(fn$)
  IF lfc_images(oim, 0) MOD 8 <> 0 THEN
    PRINT "ERROR: Object "; lfc_str$(doobj); " width is not divisble by 8!"
    END
  END IF
  'IF doobj = 1 THEN
  '  IF lfc_images(oim, 0) <> 48 OR lfc_images(oim, 1) < 25 THEN
  '    PRINT "ERROR: Window must be exactly 48 pixels wide and at least 25 pixels tall!"
  '    END
  '  END IF
  'END IF
  PRINT "  Encoding object "; lfc_str$(doobj); "..."
	
  if horzframe = 0 then
    lfr_set_obj_values_from_image oim, putterdat2
  else
    lfr_set_obj_values_from_image_hf oim, putterdat2
  end if
  
  lfr_set_obj_deriv_values putterdat2, palbit
  if dbmode = 1 then Print "  Set image-based and derived values"
  a = VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_preview_frame"))
  lfr_set_preview_image a, putterdat2
  if dbmode = 1 then Print "  Set preview frame"
	if VAL(lfc_get_ini_value$(ininum, "multi_windows")) = 0 then
    IF doobj = 1 or VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_type")) = 23 THEN lfr_set_window_values putterdat2
	else
	  IF VAL(lfc_get_ini_value$(ininum, "object_" + lfc_str$(doobj) + "_trigger_type")) = 23 then lfr_set_window_values putterdat2
	end if
  IF doobj = 0 AND palels >= 16 THEN putterdat2.animation_flags = putterdat2.animation_flags + 256
  IF doobj = 0 AND aso = 1 THEN putterdat2.animation_flags = putterdat2.animation_flags + 1024
	if doobj = 0 then putterdat2.animation_flags = putterdat2.animation_flags + 8192
  
  if dbmode = 1 then Print "  Set all metadata. Processing image..."
  
  FOR q = 1 TO putterdat2.end_animation_frame_index
    if dbmode = 1 then Print "    Processing frame" + str$(q)
	if horzframe = 0 then
      tmoim = lfc_sect_image(oim, 0, ((q - 1) * putterdat2.height), putterdat2.width, putterdat2.height)
	else
	  tmoim = lfc_sect_image(oim, ((q - 1) * putterdat2.width), 0, putterdat2.width, putterdat2.height)
	end if
	  if dbmode = 1 then Print "      Sected"
    ogposit = lfg_encode_image(tmoim, ogfile, ogposit, palbit, 0)
	  if dbmode = 1 then Print "      Encoded image"
    ogposit = lfg_encode_image(tmoim, ogfile, ogposit, 1, 1)
	  if dbmode = 1 then Print "      Encoded mask"
    lfc_close_image tmoim
	  if dbmode = 1 then Print "      Closed temporary image"
  NEXT

  lfr_put_object grfile, doobj, putterdat2
  if dbmode = 1 then Print "  Inserted metadata"
  lfc_close_image oim
  if dbmode = 1 then Print "  Closed image"
NEXT

IF adjusttriggers = 1 THEN
  lfr_get_object grfile, 0, putterdat2
  putterdat2.animation_flags = putterdat2.animation_flags + 512
  lfr_put_object grfile, 0, putterdat2
END IF

if palrgb = 1 then
  lfr_get_object grfile, 0, putterdat2
  putterdat2.animation_flags = putterdat2.animation_flags + 16384
  lfr_put_object grfile, 0, putterdat2
end if

IF VAL(lfc_get_ini_value$(ininum, "xmas_palette")) <> 0 THEN
  lfr_get_object grfile, 0, putterdat2
  putterdat2.animation_flags = putterdat2.animation_flags + 2048
  lfr_put_object grfile, 0, putterdat2
END IF

if VAL(lfc_get_ini_value$(ininum, "protect_style")) <> 0 then
  lfr_get_object grfile, 0, putterdat2
  putterdat2.animation_flags = putterdat2.animation_flags + 4096
  lfr_put_object grfile, 0, putterdat2
END IF

if VAL(lfc_get_ini_value$(ininum, "multi_windows")) <> 0 then
  lfr_get_object grfile, 0, putterdat2
  putterdat2.animation_flags = putterdat2.animation_flags + 32768
  lfr_put_object grfile, 0, putterdat2
END IF


PRINT "Saving files to disk..."

if lfc_files(grfile).size > 1056 then
  lfc_files(grfile).size = (1056 * 2)
else
  lfc_files(grfile).size = 1056
end if

if stidnam$ = "" then
  vgagrname$ = inname$ + "vgagr" + stidnum$ + ".dat"
else
  vgagrname$ = "v_" + stidnam$ + ".dat"
end if

vgagrfile = lfc_new_file(vgagrname$)
lfd_encode_file vgagrfile, 0, tgfile, 0
lfd_encode_file vgagrfile, 1, ogfile, 0

lfc_save_file vgagrfile
lfc_save_file grfile
lfc_close_file grfile
lfc_close_file tgfile
lfc_close_file ogfile
lfc_close_file vgagrfile
lfc_close_ini ininum

Print "Done!"

END