' Lemmings Functions - DAT File Manager
' Footer File
' by Namida Verasche
' see lf_dat_head.bas for info

SUB lfd_getheader (filenum, posit, varb AS lfd_SectionHeader)
varb.num_bits_in_first_byte = lfc_get_byte(filenum, posit)
varb.checksum = lfc_get_byte(filenum, posit + 1)
varb.unused1 = lfc_get_short(filenum, posit + 2)
varb.decompressed_data_size = lfc_get_short(filenum, posit + 4)
varb.unused2 = lfc_get_short(filenum, posit + 6)
varb.compressed_data_size = lfc_get_short(filenum, posit + 8)
END SUB

SUB lfd_putheader (filenum, posit, varb AS lfd_SectionHeader)
lfc_put_byte filenum, posit, varb.num_bits_in_first_byte
lfc_put_byte filenum, posit + 1, varb.checksum
lfc_put_short filenum, posit + 2, varb.unused1
lfc_put_short filenum, posit + 4, varb.decompressed_data_size
lfc_put_short filenum, posit + 6, varb.unused2
lfc_put_short filenum, posit + 8, varb.compressed_data_size
END SUB

FUNCTION lfd_extract_section (filenum, section)
k = lfc_new_file("newfile")
IF k = -1 THEN
  lfd_extract_section = -1
  EXIT FUNCTION
END IF
lfc_rename_file k, "t" + lfc_str$(k)
  
DIM decoded(0) AS _UNSIGNED _BYTE
  
lfd_decode_section filenum, section, decoded()
  
lfc_put_array k, 0, decoded()
lfd_extract_section = k
END FUNCTION


FUNCTION lfd_get_compress_size(inhead AS lfd_SectionHeader)
lfd_get_compress_size = (inhead.unused2 * 65536) + (inhead.compressed_data_size)
END FUNCTION

FUNCTION lfd_get_decompress_size(inhead AS lfd_SectionHeader)
lfd_get_decompress_size = (inhead.unused1 * 65536) + (inhead.decompressed_data_size)
END FUNCTION


FUNCTION lfd_section_location (filenum, section)
DIM tempheader AS lfd_SectionHeader
secnum = 0
posit = 0
WHILE secnum < section
  lfd_getheader filenum, posit, tempheader
  secnum = secnum + 1
  posit = posit + lfd_get_compress_size(tempheader)
  IF lfd_get_compress_size(tempheader) = 0 THEN
    lfd_section_location = posit
    EXIT FUNCTION
  END IF
WEND
lfd_section_location = posit
END FUNCTION

function lfd_section_count (filenum)
dim tempheader as lfd_SectionHeader
secct = 0
posit = 0
while posit < lfc_files(filenum).size
  lfd_getheader filenum, posit, tempheader
  secct = secct + 1
  posit = posit + lfd_get_compress_size(tempheader)
  if lfd_get_compress_size(tempheader) = 0 then
    secct = secct - 1
    exit while
  end if
wend
lfd_section_count = secct
end function

SUB lfd_decode_section (filenum, section, arry() AS _UNSIGNED _BYTE)

posit = lfd_section_location(filenum, section)
IF posit = -1 THEN EXIT SUB
  
DIM bs(0) AS _UNSIGNED _BYTE
lfd_makebitstream filenum, posit, bs()
  
lfd_decompress bs(), arry(), filenum, section
  
END SUB

SUB lfd_decompress (bs() AS _UNSIGNED _BYTE, arry() AS _UNSIGNED _BYTE, filenum, section)
  
DIM copydata(1) AS INTEGER

DIM temphead AS lfd_SectionHeader
lfd_getheader filenum, lfd_section_location(filenum, section), temphead
  
REDIM arry(lfd_get_decompress_size(temphead) - 1) AS _UNSIGNED _BYTE

cbyte = lfd_get_decompress_size(temphead) - 1
  
cbit = 0
WHILE cbyte >= 0
  copydata(0) = 0
  SELECT CASE lfd_get_bits_number(bs(), 2, cbit)
    CASE 0
      numbytes = lfd_get_bits_number(bs(), 3, cbit)
      FOR x = 0 TO numbytes
        arry(cbyte) = lfd_get_bits_byte(bs(), cbit)
        cbyte = cbyte - 1
      NEXT
    CASE 1
      copydata(0) = 2
      copydata(1) = lfd_get_bits_byte(bs(), cbit)
    CASE 2
      SELECT CASE lfd_get_bits_number(bs(), 1, cbit)
        CASE 0
          copydata(0) = 3
          copydata(1) = lfd_get_bits_number(bs(), 9, cbit)
        CASE 1
          copydata(0) = 4
          copydata(1) = lfd_get_bits_number(bs(), 10, cbit)
      END SELECT
    CASE 3
      SELECT CASE lfd_get_bits_number(bs(), 1, cbit)
        CASE 0
          copydata(0) = lfd_get_bits_byte(bs(), cbit) + 1
          copydata(1) = lfd_get_bits_number(bs(), 12, cbit)
        CASE 1
          numbytes = lfd_get_bits_byte(bs(), cbit) + 8
          FOR x = 0 TO numbytes
            arry(cbyte) = lfd_get_bits_byte(bs(), cbit)
            cbyte = cbyte - 1
          NEXT
      END SELECT
  END SELECT

  FOR x = copydata(0) TO 1 STEP -1
    arry(cbyte) = arry(cbyte + copydata(1) + 1)
    cbyte = cbyte - 1
  NEXT
WEND

END SUB

SUB lfd_shift_array (arry() AS _UNSIGNED _BYTE)
k = lfc_array_size(arry())
REDIM _PRESERVE arry(k) AS _UNSIGNED _BYTE
FOR x = k TO 1 STEP -1
  arry(x) = arry(x - 1)
NEXT
arry(0) = 0
END SUB

SUB lfd_makebitstream (filenum, posit, bs() AS _UNSIGNED _BYTE)
DIM temphead AS lfd_SectionHeader
lfd_getheader filenum, posit, temphead
bssize = lfd_bitstream_size(temphead)
cbyte = posit + lfd_get_compress_size(temphead) - 1
REDIM bs(bssize) AS _UNSIGNED _BYTE
cbit = 0
IF temphead.num_bits_in_first_byte = 0 THEN cbyte = cbyte - 1

DIM tb AS _UNSIGNED _BYTE

FOR cbyte = cbyte TO (posit + 10) STEP -1
  tb = lfc_get_byte(filenum, cbyte)
  FOR x = 0 TO 7
    bs(cbit) = 0
    IF (2 ^ x) AND tb THEN bs(cbit) = 1
    cbit = cbit + 1
    IF cbit = temphead.num_bits_in_first_byte THEN x = 7
  NEXT
NEXT
  
END SUB

SUB lfd_encode_file (filenum, section, source, overwrite)
k = lfc_files(source).size
DIM tarry(k - 1) AS _UNSIGNED _BYTE
lfc_get_array source, 0, tarry()
lfd_encode_section filenum, section, tarry(), overwrite
END SUB
  

SUB lfd_encode_section (filenum, section, arry() AS _UNSIGNED _BYTE, overwrite)
IF overwrite = 0 OR overwrite = false THEN
  overwrite = 0
ELSE
  overwrite = 1
END IF
DIM temphead AS lfd_SectionHeader
posit = lfd_section_location(filenum, section)
DIM bs(0) AS _UNSIGNED _BYTE
lfd_compress arry(), bs(), temphead
IF overwrite = 1 THEN lfd_delete_section filenum, section
lfd_make_section_space filenum, section, lfd_get_compress_size(temphead)
lfd_putheader filenum, posit, temphead
lfc_put_array filenum, posit + 10, arry()
END SUB

SUB lfd_delete_section (filenum, section)
IF lfd_section_location(filenum, section) >= lfc_files(filenum).size THEN EXIT SUB
lfc_shift_bytes filenum, lfd_section_location(filenum, section + 1), lfd_section_location(filenum, section)
END SUB

SUB lfd_make_section_space (filenum, section, newlen)
IF lfd_section_location(filenum, section) >= lfc_files(filenum).size THEN EXIT SUB
lfc_shift_bytes filenum, lfd_section_location(filenum, section), lfd_section_location(filenum, section) + newlen
END SUB

SUB lfd_compress (arry() AS _UNSIGNED _BYTE, bs() AS _UNSIGNED _BYTE, headr AS lfd_SectionHeader)

insize = lfc_array_size(arry())
maxpos = insize - 1
REDIM bs(insize * 4) AS _UNSIGNED _BYTE
  
cbyte = maxpos
  
DIM buffer(263) AS _UNSIGNED _BYTE
bfpos = 0
  
cbit = 0
extended = 0
  
  
FOR cbyte = cbyte TO 0 STEP -1
  IF cbit > (insize * extended * 4) - (insize * 2) THEN
    extended = extended + 1
    REDIM _PRESERVE bs(insize * extended * 4) AS _UNSIGNED _BYTE
  END IF
  match = 0
  matchlen = 0

  
    FOR checkoff = 1 TO (maxpos - cbyte)
      IF arry(cbyte) = arry(cbyte + checkoff) THEN
        clen = 1
        FOR bk = (cbyte - 1) TO 2 STEP -1
          IF arry(bk) = arry(bk + checkoff) THEN
            clen = clen + 1
          ELSE
            EXIT FOR
          END IF
          IF clen = 256 THEN EXIT FOR
        NEXT
        IF clen > matchlen THEN
          matchlen = clen
          match = checkoff
        END IF
      END IF
      IF matchlen = 256 THEN EXIT FOR
      IF checkoff = 4096 THEN EXIT FOR
    NEXT
 
  IF matchlen > 1 THEN
    lfd_empty_compression_buffer bfpos, buffer(), cbit, bs()
    matchlen = matchlen - 1
    match = match - 1
    handle = matchlen
    IF handle = 1 AND match > 255 THEN handle = 4
    IF handle = 2 AND match > 511 THEN handle = 4
    IF handle = 3 AND match > 1023 THEN handle = 4
    IF match > 4095 THEN handle = 0
    SELECT CASE handle
      CASE 0
        buffer(bfpos) = arry(cbyte)
        bfpos = bfpos + 1
      CASE 1
        lfd_make_bits_number 1, cbit, bs(), 2
        lfd_make_bits_number match, cbit, bs(), 8
        cbyte = cbyte - matchlen
      CASE 2
        lfd_make_bits_number 4, cbit, bs(), 3
        lfd_make_bits_number match, cbit, bs(), 9
        cbyte = cbyte - matchlen
      CASE 3
        lfd_make_bits_number 5, cbit, bs(), 3
        lfd_make_bits_number match, cbit, bs(), 10
        cbyte = cbyte - matchlen
      CASE ELSE
        lfd_make_bits_number 6, cbit, bs(), 3
        lfd_make_bits_number matchlen, cbit, bs(), 8
        lfd_make_bits_number match, cbit, bs(), 12
        cbyte = cbyte - matchlen
    END SELECT
  ELSE
    buffer(bfpos) = arry(cbyte)
    bfpos = bfpos + 1
    IF bfpos = 263 THEN lfd_empty_compression_buffer bfpos, buffer(), cbit, bs()
  END IF
  if cbyte = 8 then lfd_empty_compression_buffer bfpos, buffer(), cbit, bs()
NEXT
  
lfd_empty_compression_buffer bfpos, buffer(), cbit, bs()
  
REDIM _PRESERVE bs(cbit - 1) AS _UNSIGNED _BYTE
  
bitlen = cbit
headr.num_bits_in_first_byte = bitlen MOD 8
headr.compressed_data_size = (INT(bitlen / 8) + 11) mod 65536
headr.unused2 = int((INT(bitlen / 8) + 11) / 65536)
headr.decompressed_data_size = insize mod 65536
headr.unused1 = int(insize / 65536)
  
lfd_make_bytestream bs(), arry(), headr
  
headr.checksum = lfd_checksum(arry())
  
END SUB

sub lfd_empty_compression_buffer(bfpos, buffer() as _unsigned _byte, cbit, bs() as _unsigned _byte)
  if bfpos = 0 then exit sub
  bfpos = bfpos - 1
  IF bfpos > 7 THEN
    lfd_make_bits_number 7, cbit, bs(), 3
    lfd_make_bits_number (bfpos - 8), cbit, bs(), 8
  ELSE
    lfd_make_bits_number 0, cbit, bs(), 2
    lfd_make_bits_number (bfpos), cbit, bs(), 3
  END IF
  FOR q = 0 TO bfpos
    lfd_make_bits_byte buffer(q), cbit, bs()
  NEXT
  bfpos = 0
end sub

SUB lfd_make_bytestream (bs() AS _UNSIGNED _BYTE, arry() AS _UNSIGNED _BYTE, headr AS lfd_SectionHeader)
REDIM arry(lfd_get_compress_size(headr) - 11) AS _UNSIGNED _BYTE
cbyte = lfd_get_compress_size(headr) - 11
cbit = 0

FOR cbyte = cbyte TO 0 STEP -1
  if cbit = 0 and headr.num_bits_in_first_byte = 0 then cbyte = cbyte - 1
  FOR x = 0 TO 7
    IF bs(cbit) = 1 THEN arry(cbyte) = arry(cbyte) + 2 ^ x
    cbit = cbit + 1
    IF cbit = headr.num_bits_in_first_byte THEN x = 7
  NEXT x
NEXT

END SUB

FUNCTION lfd_checksum (arry() AS _UNSIGNED _BYTE)
mbyte = lfc_array_size(arry()) - 1
DIM result AS _UNSIGNED _BYTE
result = 0
FOR dcs = 0 TO mbyte
  result = result XOR arry(dcs)
NEXT
lfd_checksum = result
END FUNCTION

FUNCTION lfd_bitstream_size (thead AS lfd_SectionHeader)
lfd_bitstream_size = ((lfd_get_compress_size(thead) - 11) * 8) + thead.num_bits_in_first_byte
END FUNCTION

SUB lfd_make_bits_byte (value AS _UNSIGNED _BYTE, cbit, bs() AS _UNSIGNED _BYTE)
lfd_make_bits_number value, cbit, bs(), 8
END SUB

SUB lfd_make_bits_number (value AS _UNSIGNED INTEGER, cbit, bs() AS _UNSIGNED _BYTE, numbits)
FOR x = (numbits - 1) TO 0 STEP -1
  bs(cbit) = 0
  IF value AND 2 ^ x THEN bs(cbit) = 1
  cbit = cbit + 1
NEXT
END SUB

FUNCTION lfd_get_bits_byte (bs() AS _UNSIGNED _BYTE, cbit)
lfd_get_bits_byte = lfd_get_bits_number(bs(), 8, cbit)
END FUNCTION

FUNCTION lfd_get_bits_number (bs() AS _UNSIGNED _BYTE, numbits, cbit)
outnum = 0
FOR x = numbits - 1 TO 0 STEP -1
  IF bs(cbit) = 1 THEN outnum = outnum + 2 ^ x
  cbit = cbit + 1
NEXT
lfd_get_bits_number = outnum
END FUNCTION